@@ -3,12 +3,21 @@ from __future__ import print_function
 from collections import Counter, defaultdict, deque
 from functools import partial, wraps
 from heapq import merge
-from itertools import chain, count, groupby, islice, repeat, takewhile, tee
-from operator import itemgetter
+from itertools import (
+    chain,
+    compress,
+    count,
+    groupby,
+    islice,
+    repeat,
+    takewhile,
+    tee
+)
+from operator import itemgetter, lt, gt
 from sys import version_info
 
 from six import binary_type, string_types, text_type
-from six.moves import filter, map, zip, zip_longest
+from six.moves import filter, map, range, zip, zip_longest
 
 from .recipes import flatten, take
 
@@ -20,7 +29,7 @@ __all__ = [
     'collapse',
     'collate',
     'consumer',
-    'context',
+    'count_cycle',
     'distinct_permutations',
     'distribute',
     'divide',
@@ -31,6 +40,8 @@ __all__ = [
     'interleave',
     'intersperse',
     'iterate',
+    'locate',
+    'numeric_range',
     'one',
     'padded',
     'peekable',
@@ -51,38 +62,44 @@ _marker = object()
 
 
 def chunked(iterable, n):
-    """Break an iterable into lists of a given length::
+    """Break *iterable* into lists of length *n*:
 
-        >>> list(chunked([1, 2, 3, 4, 5, 6, 7], 3))
-        [[1, 2, 3], [4, 5, 6], [7]]
+        >>> list(chunked([1, 2, 3, 4, 5, 6], 3))
+        [[1, 2, 3], [4, 5, 6]]
 
-    If the length of ``iterable`` is not evenly divisible by ``n``, the last
-    returned list will be shorter.
+    If the length of *iterable* is not evenly divisible by *n*, the last
+    returned list will be shorter:
 
-    This is useful for splitting up a computation on a large number of keys
-    into batches, to be pickled and sent off to worker processes. One example
-    is operations on rows in MySQL, which does not implement server-side
-    cursors properly and would otherwise load the entire dataset into RAM on
-    the client.
+        >>> list(chunked([1, 2, 3, 4, 5, 6, 7, 8], 3))
+        [[1, 2, 3], [4, 5, 6], [7, 8]]
+
+    To use a fill-in value instead, see the :func:`grouper` recipe.
+
+    :func:`chunked` is useful for splitting up a computation on a large number
+    of keys into batches, to be pickled and sent off to worker processes. One
+    example is operations on rows in MySQL, which does not implement
+    server-side cursors properly and would otherwise load the entire dataset
+    into RAM on the client.
 
     """
     return iter(partial(take, n, iter(iterable)), [])
 
 
 def first(iterable, default=_marker):
-    """Return the first item of an iterable, ``default`` if there is none.
+    """Return the first item of *iterable*, or *default* if *iterable* is
+    empty.
 
         >>> first([0, 1, 2, 3])
         0
         >>> first([], 'some default')
         'some default'
 
-    If ``default`` is not provided and there are no items in the iterable,
+    If *default* is not provided and there are no items in the iterable,
     raise ``ValueError``.
 
-    ``first()`` is useful when you have a generator of expensive-to-retrieve
+    :func:`first` is useful when you have a generator of expensive-to-retrieve
     values and want any arbitrary one. It is marginally shorter than
-    ``next(iter(...), default)``.
+    ``next(iter(iterable), default)``.
 
     """
     try:
@@ -101,8 +118,8 @@ def first(iterable, default=_marker):
 class peekable(object):
     """Wrap an iterator to allow lookahead and prepending elements.
 
-    Call ``peek()`` on the result to get the value that will next pop out of
-    ``next()``, without advancing the iterator:
+    Call :meth:`peek` on the result to get the value that will be returned
+    by :func:`next`. This won't advance the iterator:
 
         >>> p = peekable(['a', 'b'])
         >>> p.peek()
@@ -110,15 +127,15 @@ class peekable(object):
         >>> next(p)
         'a'
 
-    Pass ``peek()`` a default value to return that instead of raising
+    Pass :meth:`peek` a default value to return that instead of raising
     ``StopIteration`` when the iterator is exhausted.
 
         >>> p = peekable([])
         >>> p.peek('hi')
         'hi'
 
-    peekables also offer a ``prepend()`` method which will insert items before
-    the remaining part of the underlying source iterator.
+    peekables also offer a :meth:`prepend` method, which "inserts" items
+    at the head of the iterable:
 
         >>> p = peekable([1, 2, 3])
         >>> p.prepend(10, 11, 12)
@@ -129,13 +146,9 @@ class peekable(object):
         >>> list(p)
         [11, 12, 1, 2, 3]
 
-    Prepended items are treated by other peekable methods exactly as if they
-    had come from the source iterator.
-
-    You may index the peekable to look ahead by more than one item.
-    The values up to the index you specified will be cached.
-    Index 0 is the item that will be returned by ``next()``, index 1 is the
-    item after that, and so on:
+    peekables can be indexed. Index 0 is the item that will be returned by
+    :func:`next`, index 1 is the item after that, and so on:
+    The values up to the given index will be cached.
 
         >>> p = peekable(['a', 'b', 'c', 'd'])
         >>> p[0]
@@ -144,27 +157,20 @@ class peekable(object):
         'b'
         >>> next(p)
         'a'
-        >>> p.prepend('x')
-        >>> p[1]
-        'b'
-        >>> next(p)
-        'x'
-        >>> next(p)
-        'b'
 
     Negative indexes are supported, but be aware that they will cache the
     remaining items in the source iterator, which may require significant
     storage.
 
-    To test whether there are more items in the iterator, examine the
-    peekable's truth value. If it is truthy, there are more items (which may
-    have been prepended or obtained from the source iterator).
+    To check whether a peekable is exhausted, check its truth value:
 
-        >>> assert peekable([1])
-        >>> p = peekable([])
-        >>> assert not p
-        >>> p.prepend(1)
-        >>> assert p
+        >>> p = peekable(['a', 'b'])
+        >>> if p:  # peekable has items
+        ...     list(p)
+        ['a', 'b']
+        >>> if not p:  # peekable is exhaused
+        ...     list(p)
+        []
 
     """
     def __init__(self, iterable):
@@ -285,7 +291,7 @@ def _collate(*iterables, **kwargs):
     key = kwargs.pop('key', lambda a: a)
     reverse = kwargs.pop('reverse', False)
 
-    min_or_max = partial(max if reverse else min, key=lambda a_b: a_b[0])
+    min_or_max = partial(max if reverse else min, key=itemgetter(0))
     peekables = [peekable(it) for it in iterables]
     peekables = [p for p in peekables if p]  # Kill empties.
     while peekables:
@@ -302,7 +308,7 @@ def collate(*iterables, **kwargs):
         ['A', 'A', 'C', 'D', 'J', 'K', 'L', 'Z', 'Z']
 
     Works lazily, keeping only the next value from each iterable in memory. Use
-    ``collate()`` to, for example, perform a n-way mergesort of items that
+    :func:`collate` to, for example, perform a n-way mergesort of items that
     don't fit in memory.
 
     :arg key: A function that returns a comparison value for an item. Defaults
@@ -315,7 +321,7 @@ def collate(*iterables, **kwargs):
     unexpected results.
 
     If neither of the keyword arguments are specified, this function delegates
-    to ``heapq.merge()``.
+    to :func:`heapq.merge`.
 
     """
     if not kwargs:
@@ -361,7 +367,7 @@ def consumer(func):
 
 
 def ilen(iterable):
-    """Return the number of items in ``iterable``.
+    """Return the number of items in *iterable*.
 
         >>> ilen(x for x in range(1000000) if x % 3 == 0)
         333334
@@ -433,7 +439,7 @@ def one(iterable):
 
 
 def distinct_permutations(iterable):
-    """Yield successive distinct permutations of the elements in the iterable.
+    """Yield successive distinct permutations of the elements in *iterable*.
 
         >>> sorted(distinct_permutations([1, 0, 1]))
         [(0, 1, 1), (1, 0, 1), (1, 1, 0)]
@@ -444,8 +450,8 @@ def distinct_permutations(iterable):
 
     Duplicate permutations arise when there are duplicated elements in the
     input iterable. The number of items returned is
-    ``n! / (x_1! * x_2! * ... * x_n!)``, where ``n`` is the total number of
-    items input, and each ``x_i`` is the count of a distinct item in the input
+    `n! / (x_1! * x_2! * ... * x_n!)`, where `n` is the total number of
+    items input, and each `x_i` is the count of a distinct item in the input
     sequence.
 
     """
@@ -481,7 +487,7 @@ def distinct_permutations(iterable):
 
 
 def intersperse(e, iterable):
-    """Intersperse element ``e`` between the elements of *iterable*.
+    """Intersperse object *e* between the items of *iterable*.
 
         >>> list(intersperse('x', 'ABCD'))
         ['A', 'x', 'B', 'x', 'C', 'x', 'D']
@@ -536,13 +542,13 @@ def windowed(seq, n, fillvalue=None, step=1):
         >>> list(all_windows)
         [(1, 2, 3), (2, 3, 4), (3, 4, 5)]
 
-    When the window is larger than the iterable, ``fillvalue`` is used in place
+    When the window is larger than the iterable, *fillvalue* is used in place
     of missing values::
 
         >>> list(windowed([1, 2, 3], 4))
         [(1, 2, 3, None)]
 
-    Each window will advance in increments of *step*::
+    Each window will advance in increments of *step*:
 
         >>> list(windowed([1, 2, 3, 4, 5, 6], 3, fillvalue='!', step=2))
         [(1, 2, 3), (3, 4, 5), (5, 6, '!')]
@@ -582,8 +588,8 @@ def windowed(seq, n, fillvalue=None, step=1):
 
 
 class bucket(object):
-    """Wrap an iterable and return an object that buckets the iterable into
-    child iterables based on a ``key`` function.
+    """Wrap *iterable* and return an object that buckets it iterable into
+    child iterables based on a *key* function.
 
         >>> iterable = ['a1', 'b1', 'c1', 'a2', 'b2', 'c2', 'b3']
         >>> s = bucket(iterable, key=lambda s: s[0])
@@ -597,6 +603,7 @@ class bucket(object):
 
     The original iterable will be advanced and its items will be cached until
     they are used by the child iterables. This may require significant storage.
+
     Be aware that attempting to select a bucket that no items correspond to
     will exhaust the iterable and cache all values.
 
@@ -688,20 +695,21 @@ def spy(iterable, n=1):
 
 def interleave(*iterables):
     """Return a new iterable yielding from each iterable in turn,
-    until the shortest is exhausted. Note that this is the same as
-    ``chain(*zip(*iterables))``.
+    until the shortest is exhausted.
 
         >>> list(interleave([1, 2, 3], [4, 5], [6, 7, 8]))
         [1, 4, 6, 2, 5, 7]
 
+    For a version that doesn't terminate after the shortest iterable is
+    exhausted, see :func:`interleave_longest`.
+
     """
     return chain.from_iterable(zip(*iterables))
 
 
 def interleave_longest(*iterables):
     """Return a new iterable yielding from each iterable in turn,
-    skipping any that are exhausted. Note that this is not the same as
-    ``chain(*zip_longest(*iterables))``.
+    skipping any that are exhausted.
 
         >>> list(interleave_longest([1, 2, 3], [4, 5], [6, 7, 8]))
         [1, 4, 6, 2, 5, 7, 3, 8]
@@ -712,17 +720,27 @@ def interleave_longest(*iterables):
 
 
 def collapse(iterable, base_type=None, levels=None):
-    """Flatten an iterable containing some iterables (themselves containing
-    some iterables, etc.) into non-iterable types, strings, elements
-    matching ``isinstance(element, base_type)``, and elements that are
-    ``levels`` levels down.
+    """Flatten an iterable with multiple levels of nesting (e.g., a list of
+    lists of tuples) into non-iterable types.
 
-        >>> list(collapse([[1], 2, [[3], 4], [[[5]]], 'abc']))
-        [1, 2, 3, 4, 5, 'abc']
-        >>> list(collapse([[1], 2, [[3], 4], [[[5]]]], levels=2))
-        [1, 2, 3, 4, [5]]
-        >>> list(collapse((1, [2], (3, [4, (5,)])), list))
-        [1, [2], 3, [4, (5,)]]
+        >>> iterable = [(1, 2), ([3, 4], [[5], [6]])]
+        >>> list(collapse(iterable))
+        [1, 2, 3, 4, 5, 6]
+
+    String types are not considered iterable and will not be collapsed.
+    To avoid collapsing other types, specify *base_type*:
+
+        >>> iterable = ['ab', ('cd', 'ef'), ['gh', 'ij']]
+        >>> list(collapse(iterable, base_type=tuple))
+        ['ab', ('cd', 'ef'), 'gh', 'ij']
+
+    Specify *levels* to stop flattening after a certain level:
+
+    >>> iterable = [('a', ['b']), ('c', ['d'])]
+    >>> list(collapse(iterable))  # Fully flattened
+    ['a', 'b', 'c', 'd']
+    >>> list(collapse(iterable, levels=1))  # Only one level flattened
+    ['a', ['b'], 'c', ['d']]
 
     """
     def walk(node, level):
@@ -748,13 +766,16 @@ def collapse(iterable, base_type=None, levels=None):
         yield x
 
 
-def side_effect(func, iterable, chunk_size=None):
+def side_effect(func, iterable, chunk_size=None, before=None, after=None):
     """Invoke *func* on each item in *iterable* (or on each *chunk_size* group
     of items) before yielding the item.
 
     `func` must be a function that takes a single argument. Its return value
     will be discarded.
 
+    *before* and *after* are optional functions that take no arguments. They
+    will be executed before iteration starts and after it ends, respectively.
+
     `side_effect` can be used for logging, updating progress bars, or anything
     that is not functionally "pure."
 
@@ -779,23 +800,32 @@ def side_effect(func, iterable, chunk_size=None):
 
         >>> from io import StringIO
         >>> from more_itertools import consume
-        >>> with StringIO() as f:
-        ...     func = lambda x: print(x, end=u',', file=f)
-        ...     it = [u'a', u'b', u'c']
-        ...     consume(side_effect(func, it))
-        ...     print(f.getvalue())
-        a,b,c,
+        >>> f = StringIO()
+        >>> func = lambda x: print(x, file=f)
+        >>> before = lambda: print(u'HEADER', file=f)
+        >>> after = f.close
+        >>> it = [u'a', u'b', u'c']
+        >>> consume(side_effect(func, it, before=before, after=after))
+        >>> f.closed
+        True
 
     """
-    if chunk_size is None:
-        for item in iterable:
-            func(item)
-            yield item
-    else:
-        for chunk in chunked(iterable, chunk_size):
-            func(chunk)
-            for item in chunk:
+    try:
+        if before is not None:
+            before()
+
+        if chunk_size is None:
+            for item in iterable:
+                func(item)
                 yield item
+        else:
+            for chunk in chunked(iterable, chunk_size):
+                func(chunk)
+                for item in chunk:
+                    yield item
+    finally:
+        if after is not None:
+            after()
 
 
 def sliced(seq, n):
@@ -810,8 +840,8 @@ def sliced(seq, n):
         >>> list(sliced((1, 2, 3, 4, 5, 6, 7, 8), 3))
         [(1, 2, 3), (4, 5, 6), (7, 8)]
 
-    This function will only work for sliceable objects. For non-sliceable
-    iterable, see ``chunked()``.
+    This function will only work for iterables that support slicing.
+    For non-sliceable iterables, see :func:`chunked`.
 
     """
     return takewhile(bool, (seq[i: i + n] for i in count(0, n)))
@@ -900,23 +930,23 @@ def distribute(n, iterable):
         >>> list(group_2)
         [2, 4, 6]
 
-    If the length of the iterable is not evenly divisible by n, then the
-    length of the smaller iterables will not be identical::
+    If the length of *iterable* is not evenly divisible by *n*, then the
+    length of the returned iterables will not be identical:
 
         >>> children = distribute(3, [1, 2, 3, 4, 5, 6, 7])
         >>> [list(c) for c in children]
         [[1, 4, 7], [2, 5], [3, 6]]
 
-    If the length of the iterable is smaller than n, then the last returned
+    If the length of *iterable* is smaller than *n*, then the last returned
     iterables will be empty:
 
         >>> children = distribute(5, [1, 2, 3])
         >>> [list(c) for c in children]
         [[1], [2], [3], [], []]
 
-    This function uses ``itertools.tee`` and may require significant storage.
-    If you need the order items in the smaller iterables to match the original
-    iterable, see ``divide()``.
+    This function uses :func:`itertools.tee` and may require significant
+    storage. If you need the order items in the smaller iterables to match the
+    original iterable, see :func:`divide`.
 
     """
     if n < 1:
@@ -928,8 +958,8 @@ def distribute(n, iterable):
 
 def stagger(iterable, offsets=(-1, 0, 1), longest=False, fillvalue=None):
     """Yield tuples whose elements are offset from *iterable*.
-    The amount by which the ``i``-th item in each tuple is offset is given by
-    the ``i``-th item in *offsets*.
+    The amount by which the `i`-th item in each tuple is offset is given by
+    the `i`-th item in *offsets*.
 
         >>> list(stagger([0, 1, 2, 3]))
         [(None, 0, 1), (0, 1, 2), (1, 2, 3)]
@@ -955,8 +985,8 @@ def stagger(iterable, offsets=(-1, 0, 1), longest=False, fillvalue=None):
 
 
 def zip_offset(*iterables, **kwargs):
-    """``zip`` the input *iterables* together, but offset the ``i``-th iterable
-    by the ``i``-th item in *offsets*.
+    """``zip`` the input *iterables* together, but offset the `i`-th iterable
+    by the `i`-th item in *offsets*.
 
         >>> list(zip_offset('0123', 'abcdef', offsets=(0, 1)))
         [('0', 'b'), ('1', 'c'), ('2', 'd'), ('3', 'e')]
@@ -1040,8 +1070,8 @@ def divide(n, iterable):
         >>> list(group_2)
         [4, 5, 6]
 
-    If the length of the iterable is not evenly divisible by n, then the
-    length of the smaller iterables will not be identical::
+    If the length of *iterable* is not evenly divisible by *n*, then the
+    length of the returned iterables will not be identical:
 
         >>> children = divide(3, [1, 2, 3, 4, 5, 6, 7])
         >>> [list(c) for c in children]
@@ -1055,7 +1085,7 @@ def divide(n, iterable):
         [[1], [2], [3], [], []]
 
     This function will exhaust the iterable before returning and may require
-    significant storage. If order is not important, see ``distribute()``,
+    significant storage. If order is not important, see :func:`distribute`,
     which does not first pull the iterable into memory.
 
     """
@@ -1125,8 +1155,8 @@ def always_iterable(obj):
 
 
 def adjacent(predicate, iterable, distance=1):
-    """Return an iterable over ``(bool, item)`` tuples where the ``item`` is
-    drawn from *iterable* and the ``bool`` indicates whether
+    """Return an iterable over `(bool, item)` tuples where the `item` is
+    drawn from *iterable* and the `bool` indicates whether
     that item satisfies the *predicate* or is adjacent to an item that does.
 
     For example, to find whether items are adjacent to a ``3``::
@@ -1134,7 +1164,7 @@ def adjacent(predicate, iterable, distance=1):
         >>> list(adjacent(lambda x: x == 3, range(6)))
         [(False, 0), (False, 1), (True, 2), (True, 3), (True, 4), (False, 5)]
 
-    Set *distance* to change what counts as adjacent. For example, To find
+    Set *distance* to change what counts as adjacent. For example, to find
     whether items are two places away from a ``3``:
 
         >>> list(adjacent(lambda x: x == 3, range(6), distance=2))
@@ -1148,8 +1178,8 @@ def adjacent(predicate, iterable, distance=1):
     The predicate function will only be called once for each item in the
     iterable.
 
-    See also ``groupby_transform()``, which can be used with this function
-    to group ranges of items with the same ``bool`` value.
+    See also :func:`groupby_transform`, which can be used with this function
+    to group ranges of items with the same `bool` value.
 
     """
     # Allow distance=0 mainly for testing that it reproduces results with map()
@@ -1164,26 +1194,23 @@ def adjacent(predicate, iterable, distance=1):
 
 
 def groupby_transform(iterable, keyfunc=None, valuefunc=None):
-    """Make an iterator that returns consecutive keys and groups from the
-    *iterable*. *keyfunc* is a function used to compute a grouping key for each
-    item. *valuefunc* is a function for transforming the items after grouping.
+    """An extension of :func:`itertools.groupby` that transforms the values of
+    *iterable* after grouping them.
+    *keyfunc* is a function used to compute a grouping key for each item.
+    *valuefunc* is a function for transforming the items after grouping.
+
+        >>> iterable = 'AaaABbBCcA'
+        >>> keyfunc = lambda x: x.upper()
+        >>> valuefunc = lambda x: x.lower()
+        >>> grouper = groupby_transform(iterable, keyfunc, valuefunc)
+        >>> [(k, ''.join(g)) for k, g in grouper]
+        [('A', 'aaaa'), ('B', 'bbb'), ('C', 'cc'), ('A', 'a')]
 
     *keyfunc* and *valuefunc* default to identity functions if they are not
-    specified. When *valuefunc* is not specified, ``groupby_transform`` is the
-    same as ``itertools.groupby()``.
+    specified.
 
-    For example, to group a list of numbers by rounding down to the nearest 10,
-    and then transform them into strings:
-
-        >>> iterable = [0, 1, 12, 13, 23, 24]
-        >>> keyfunc = lambda x: 10 * (x // 10)
-        >>> valuefunc = lambda x: str(x)
-        >>> grouper = groupby_transform(iterable, keyfunc, valuefunc)
-        >>> [(k, list(g)) for k, g in grouper]
-        [(0, ['0', '1']), (10, ['12', '13']), (20, ['23', '24'])]
-
-    ``groupby_transform`` is useful when grouping elements of an iterable using
-    a separate iterable as the key. To do this,  ``zip()`` the iterables
+    :func:`groupby_transform` is useful when grouping elements of an iterable
+    using a separate iterable as the key. To do this, :func:`zip` the iterables
     and pass a *keyfunc* that extracts the first element and a *valuefunc*
     that extracts the second element::
 
@@ -1200,37 +1227,97 @@ def groupby_transform(iterable, keyfunc=None, valuefunc=None):
     return ((k, map(valuefunc, g)) for k, g in groupby(iterable, keyfunc))
 
 
-def context(obj):
-    """Wrap *obj*, an object that supports the context manager protocol,
-    in a ``with`` statement and then yield the resultant object.
+def numeric_range(*args):
+    """An extension of the built-in ``range()`` function whose arguments can
+    be any orderable numeric type.
 
-    The object's ``__enter__()`` method runs before this function yields, and
-    its ``__exit__()`` method runs after control returns to this function.
+    With only *stop* specified, *start* defaults to ``0`` and *step*
+    defaults to ``1``. The output items will match the type of *stop*:
 
-    This can be used to operate on objects that can close automatically when
-    using a ``with`` statement, like IO objects:
+        >>> list(numeric_range(3.5))
+        [0.0, 1.0, 2.0, 3.0]
 
-        >>> from io import StringIO
-        >>> from more_itertools import consume
-        >>> it = [u'1', u'2', u'3']
-        >>> file_obj = StringIO()
-        >>> consume(print(x, file=f) for f in context(file_obj) for x in it)
-        >>> file_obj.closed
-        True
+    With only *start* and *stop* specified, *step* defaults to ``1``. The
+    output items will match the type of *start*:
 
-    Be sure to iterate over the returned context manager in the outermost
-    loop of a nested loop structure so it only enters and exits once::
+        >>> from decimal import Decimal
+        >>> start = Decimal('2.1')
+        >>> stop = Decimal('5.1')
+        >>> list(numeric_range(start, stop))
+        [Decimal('2.1'), Decimal('3.1'), Decimal('4.1')]
 
-        >>> # Right
-        >>> file_obj = StringIO()
-        >>> consume(print(x, file=f) for f in context(file_obj) for x in it)
+    With *start*, *stop*, and *step*  specified the output items will match
+    the type of ``start + step``:
+
+        >>> from fractions import Fraction
+        >>> start = Fraction(1, 2)  # Start at 1/2
+        >>> stop = Fraction(5, 2)  # End at 5/2
+        >>> step = Fraction(1, 2)  # Count by 1/2
+        >>> list(numeric_range(start, stop, step))
+        [Fraction(1, 2), Fraction(1, 1), Fraction(3, 2), Fraction(2, 1)]
+
+    If *step* is zero, ``ValueError`` is raised. Negative steps are supported:
+
+        >>> list(numeric_range(3, -1, -1.0))
+        [3.0, 2.0, 1.0, 0.0]
+
+    Be aware of the limitations of floating point numbers; the representation
+    of the yielded numbers may be surprising.
 
-        >>> # Wrong
-        >>> file_obj = StringIO()
-        >>> consume(print(x, file=f) for x in it for f in context(file_obj))
-        Traceback (most recent call last):
-        ...
-        ValueError: I/O operation on closed file.
     """
-    with obj as context_obj:
-        yield context_obj
+    argc = len(args)
+    if argc == 1:
+        stop, = args
+        start = type(stop)(0)
+        step = 1
+    elif argc == 2:
+        start, stop = args
+        step = 1
+    elif argc == 3:
+        start, stop, step = args
+    else:
+        err_msg = 'numeric_range takes at most 3 arguments, got {}'
+        raise TypeError(err_msg.format(argc))
+
+    values = (start + (step * n) for n in count())
+    if step > 0:
+        return takewhile(partial(gt, stop), values)
+    elif step < 0:
+        return takewhile(partial(lt, stop), values)
+    else:
+        raise ValueError('numeric_range arg 3 must not be zero')
+
+
+def count_cycle(iterable, n=None):
+    """Cycle through the items from *iterable* up to *n* times, yielding
+    the number of completed cycles along with each item. If *n* is omitted the
+    process repeats indefinitely.
+
+    >>> list(count_cycle('AB', 3))
+    [(0, 'A'), (0, 'B'), (1, 'A'), (1, 'B'), (2, 'A'), (2, 'B')]
+
+    """
+    iterable = tuple(iterable)
+    if not iterable:
+        return iter(())
+    counter = count() if n is None else range(n)
+    return ((i, item) for i in counter for item in iterable)
+
+
+def locate(iterable, pred=bool):
+    """Yield the index of each item in *iterable* for which *pred* returns
+    ``True``.
+
+    *pred* defaults to :func:`bool`, which will select truthy items:
+
+        >>> list(locate([0, 1, 1, 0, 1, 0, 0]))
+        [1, 2, 4]
+
+    Set *pred* to a custom function to, e.g., find the indexes for a particular
+    item:
+
+        >>> list(locate(['a', 'b', 'c', 'b'], lambda x: x == 'b'))
+        [1, 3]
+
+    """
+    return compress(count(), map(pred, iterable))
